#include "snapshots.h"
#include <stdlib.h>

/*
 * u8 *OpenSNAfile ( LPSTR szFilename )
 *
 * 
 *
 */
u8 *OpenSNAfile ( LPSTR szFilename )
{
sna_t*	pSNA;
u8		*memory;

	/* load the snapshot file */
	if ( (pSNA = (sna_t*) FILE_Load( szFilename )) == NULL )
		return (NULL);
	
	/* allocate an memory image */
	if ( (memory=(u8*)malloc(MEMORY_SIZE)) == NULL ) {
		FILE_Unload( pSNA );
		return(NULL);
	}
	memset( memory, 0x00, MEMORY_SIZE );

	/* stick the SNA file into the image */
	memcpy( &memory[0x4000], pSNA->image, SNA_IMAGE_SIZE );

	/* dump the SNA file */
	FILE_Unload( pSNA );

	return ( memory );
}

/*
 * void *FILE_Load( LPSTR szFilename)
 * Allocates memory for the file and reads the entire contents into
 * memory.
 *
 * ENTRY:       LPSTR szFilename - File to get!
 * EXIT:        void *          - pointer to file in memory
 *
 * NOTE:        Remember to free the memory!
 */

void *FILE_Load( LPSTR szFilename)
{
FILE *infile;
size_t length;
void *buffer;

	if ( (infile = fopen( szFilename, "rb" )) == NULL )
		return NULL;

	length = FILE_Length ( infile );

	if( (buffer = malloc( length )) == NULL ) {
		fclose( infile );
		return NULL;
	}

	if ( (fread( buffer, sizeof(u8), length, infile )) != length )	{
		fclose( infile );
		free( buffer );
		return NULL;
	}
	
	fclose( infile );

	return ( buffer );
}

void FILE_Unload ( void *mem )
{
	free ( mem );
}

/*
 * size_t FILE_Length ( FILE *infile )
 * Returns the length of the file pointed to by the stream.
 *
 * ENTRY:       FILE * infile   - File stream handle
 * EXIT:        size_t          - length of file
 *
 * NOTE:        This function remembers the current file position, and thus
 *              can be called at any point during file access.
 */

size_t FILE_Length ( FILE *infile )
{
size_t length;
s32 curpos;

	/* remember current file position */
	curpos = ftell ( infile );

	/* then, move to the end of the file */
	fseek ( infile, 0, SEEK_END );

	/* giving us its length therefore! */
	length = ftell ( infile );

	/* return the file seek position to whence we came */
	fseek ( infile, curpos, SEEK_SET );

	return length;
}
